/* -*- mode: c++; c-basic-offset: 4; -*- */
#include <stdexcept>
#include "html/compound.hh"
#include "html/genAttr.hh"
#include "html/writer.hh"
#include "html/align.hh"
#include "html/style.hh"
#include "html/text.hh"

namespace html {
    class linebreak : public object {
    public:
        linebreak(void);
        virtual ~linebreak(void);
        linebreak* clone(void) const;
        void write(writer& w) const;
    };
}    

//======================================  Construct an empty compound
html::compound::compound(void) {
}

//======================================  Copy a compound object
html::compound::compound(const compound& x) 
  : mAttr(x.mAttr)
{
    for (const_html_iter i=x.mList.begin() ; i != x.mList.end(); i++) {
        addObject(**i);
    }
}

//======================================  Destroy a compound object
html::compound::~compound(void) {
}

//======================================  Add a linebreak to the compound
html::compound&
html::compound::lineBreak(void) {
    mList.push_back(new linebreak);
    return *this;
}

//======================================  Remove an object from the compound
void 
html::compound::remObject(const object* p) {
    for (html_iter i=mList.begin(); i != mList.end(); ++i) {
	if (i->get() == p) {
	    mList.erase(i);
	    return;
	}
    }
}

//======================================  Remove all objects from the compound
void 
html::compound::clear(void) {
    mList.clear();
}


//======================================  Clone an identical compound
html::compound*
html::compound::clone(void) const {
    return new compound(*this);
}

const char*
html::compound::getTag(void) const {
    return reinterpret_cast<const char*>(0);
}

//======================================  Write compound to output file
void 
html::compound::install(const char* from, const char* to) const {
    for (const_html_iter i=mList.begin() ; i != mList.end() ; i++) {
        (*i)->install(from, to);
    }
}

//======================================  Add a compound attribute
html::compound&
html::compound::operator<<(const html::attrib& att) {
    std::string atype;
   if (dynamic_cast<const html::align*>(&att) != NULL) {
      atype = "align";
   } else if (dynamic_cast<const html::styleAttrib*>(&att) != NULL) {
      atype = "style";
   } else {
       throw std::runtime_error("Non-block attribute added to compound");
   }
   mAttr.addAttr(atype, att);
   return *this;
}

//======================================  Add a text string
html::compound&
html::compound::operator<<(const std::string& txt) {
    addObject(text(txt));
    return *this;
}

//======================================  Set the class attribute.
void 
html::compound::setClass(const std::string& name) {
    addAttr("class", genAttr(name));
}

//======================================  Write compound to output file
void 
html::compound::write(writer& out) const {
    if (getTag()) out.tag(getTag(), mAttr);
    out.pushDefault(mAttr);
    for (const_html_iter i=mList.begin() ; i != mList.end() ; i++) {
        (*i)->write(out);
    }
    out.popDefault();
    if (getTag()) out.endTag(getTag());
}

//======================================  linebreak constructors
html::linebreak::linebreak(void) {}

html::linebreak::~linebreak(void) {}

html::linebreak*
html::linebreak::clone(void) const {
    return new linebreak;
}

//======================================  Write linebreak to output file
void 
html::linebreak::write(writer& out) const {
    out.tagNData("br");
}
//======================================  Inline methods.
void 
html::compound::addAttr(const std::string& name, const std::string& value) {
    mAttr.addAttr(name, genAttr(value));
}

//======================================  Remove all objects from the compound
void 
html::compound::clearAttr(void) {
    mAttr.clear();
}

html::div::div(const std::string& divclass) {
    setClass(divclass);
}
